/* adv_settings.js
======================================================================
==   Sage++ (Higmmer's Edition)                                     ==
==     Copyright (C) Higmmer.NEeT / Higmmer All Rights Reserved.    ==
==     The original Sage is the work of The Sage Project.           ==
======================================================================
*/


function AdvSettings() {
	this._dataset = {};
	this.tagFilter = []; this.attrFilter = [];
	this.schemeFilter = null;
	this.cssFilter = null;
	this.adBlockTitle = this.adBlockUrl = null;
	this.ignoreEnclosures = null;
	this.sbmUrlFilter = null;
}
AdvSettings.prototype = {
	addData: function(data) {
		this._dataset[data.id] = data;
		return this;
	},
	initAll: function() {
		for each(var data in this._dataset){
			if(typeof data.init == "function") data.init(this);
		}
	},
	acceptAll: function() {
		for each(var data in this._dataset){
			if(typeof data.accept == "function") data.accept(this);
		}
	},
	getData: function(id) {
		return this._dataset[id];
	}
};


var tabBox = strRes = null;
var settings = dataset = null;


function init() {
	tabBox = $("tabAdvSettingsBox");
	restoreLastTab();
	strRes = $("strRes");
	
	settings = new AdvSettings();
	dataset = createDataSet();
	dataset.forEach(function(data){ settings.addData(data); });
	
	settings.initAll();
	setDisabled();
}

function accept() {
	settings.acceptAll();
}

function setDisabled() {
	$("txtUriSchemeOthers").disabled = !$("chkUriSchemeOthers").checked;
}

function saveLastTab() {
	CommonFunc.setPrefValue(CommonFunc.SETTINGS_ADV_SELECTED_TAB, "int", tabBox.selectedIndex);
}

function restoreLastTab() {
	tabBox.selectedIndex = CommonFunc.getPrefValue(CommonFunc.SETTINGS_ADV_SELECTED_TAB, "int", 0);
}

function createDataSet() {
	return [
		new HtmlFilterTreeData("treeHtmlElemAttrFilter", CommonFunc.FILTER_ACCEPT_TAGS, "str"),
		new UriSchemeData("chkUriScheme-http", CommonFunc.FILTER_ACCEPT_SCHEMES, "str"),
		new UriSchemeData("chkUriScheme-https", CommonFunc.FILTER_ACCEPT_SCHEMES, "str"),
		new UriSchemeData("chkUriScheme-ftp", CommonFunc.FILTER_ACCEPT_SCHEMES, "str"),
		new UriSchemeData("txtUriSchemeOthers", CommonFunc.FILTER_ACCEPT_SCHEMES, "str"),
		new CheckboxData("chkFilterDebugMode", CommonFunc.FILTER_DEBUG_MODE, "bool", false),
		new CssFilterListData("listCssFilter", CommonFunc.FILTER_CSS_REJECT_REGEXP, "str"),
		new AdBlockTitleListData("listAdBlockTitle", CommonFunc.FILTER_ADBLOCK_TITLE, "wstr"),
		new AdBlockUrlListData("listAdBlockUrl", CommonFunc.FILTER_ADBLOCK_REGEXP, "str"),
		new IgnoreEnclosureListData("listIgnoteEnclosure", CommonFunc.ENCLOSURE_IGNORE_FILETYPES, "str"),
		new CheckboxData("chkEnclosureIconize", CommonFunc.ENCLOSURE_ICONIZE, "bool", true),
		new SbmAddUrlTextboxData("txtSbmAddUrl", CommonFunc.SBM_ADD_URL, "str", CreateHTML.getSbmAddUrlDefault),
		new SbmUrlFilterListData("listSbmUrlFilter", CommonFunc.SBM_URL_FILTER, "str")
	];
}


//---------- Event Handlers ----------

function handleClickAdd(event) {
	var targetId = getTargetId(event);
	if(targetId){
		var data = settings.getData(targetId);
		if(data.addNewItem(settings)) data.rebuildData(settings);
		$(targetId).focus();
	}
}

function handleClickEdit(event) {
	var targetId = getTargetId(event);
	if(targetId){
		var data = settings.getData(targetId);
		var selectedItem = data.getSelectedItem();
		if(selectedItem){
			if(data.editItem(selectedItem, settings)) data.rebuildData(settings);
		}
		$(targetId).focus();
	}
}

function handleClickRemove(event) {
	var targetId = getTargetId(event);
	if(targetId){
		var data = settings.getData(targetId);
		var selectedItem = data.getSelectedItem();
		if(selectedItem){
			data.removeItem(selectedItem);
			data.rebuildData(settings);
		}
		$(targetId).focus();
	}
}

var handlers = [ handleClickAdd, handleClickEdit, handleClickRemove ];
function handleKeyUp(event) {
	if(!event.altKey) return;
	var key = String.fromCharCode(event.keyCode).toUpperCase();
	
	var dispatcher = null, btn = null;
	var btns = [ $("btnAdd_" + event.target.id), $("btnEdit_" + event.target.id), $("btnRemove_" + event.target.id) ];
	
	for(var i = 0; i < btns.length; i++){
		if(btns[i].accessKey == key){
			btn = btns[i]; dispatcher = handlers[i]; break;
		}
	}
	if(!dispatcher || !btn) return;
	
	dispatcher({ target:btn });
	event.preventDefault();
}

function openEditDialog(editMode, labelRes, valueArray, descRes, validator, acceptor) {
	var title = strRes.getString(editMode ? "AS_DLG_EDIT_TITLE" : "AS_DLG_ADD_TITLE");
	var label = getStrResArray(labelRes);
	var desc = getStrResArray(descRes);
	
	window.openDialog("chrome://sagepp/content/dialogs/settings/advanced/adv_edit.xul",
	                  "sagepp_advanced_settings_edit", "chrome,modal,centerscreen",
	                  title, label, valueArray, desc, validator, acceptor);
}

function getTargetId(event) {
	return event.target.id.match(/^btn[a-zA-Z]+_(.*)$/) ? RegExp.$1 : "";
}

function getStrResArray(resArray) {
	var result = [];
	for(var i = 0; i < resArray.length; i++){
		try{ result.push(strRes.getString(resArray[i])); } catch(e){}
	}
	return result;
}


/* End of File */
